#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
    tkScenarist - screen writing made simpler

    Copyright (c) 2014+ Raphaël Seban <motus@laposte.net>

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.

    If not, see http://www.gnu.org/licenses/
"""

import locale
locale.setlocale(locale.LC_ALL, "")

import tkRAD
#~ tkRAD.i18n.switch_off()


class tkScenarist (tkRAD.RADApplication):

    APP = {
        "name": _("tkScenarist"),
        "version": _("1.3.2"),
        "description": _("Movie scriptwriting utility program."),
        "title": _("tkScenarist - screen writing made simpler"),
        "author": _("Raphaël SEBAN <motus@laposte.net>"),
        # current maintainer (may be different from author)
        "maintainer": _("Raphaël SEBAN"),
        "copyright": _("© 2014+ Raphaël SEBAN."),
        "license": _("Licensed under GNU General Public License v3."),
        "license_url": _("http://www.gnu.org/licenses/"),
        "pdflib": _("ReportLab®"),
        "pdflib_author": _("© 2000-2012 ReportLab Europe Ltd."),
    }

    DIRECTORIES = (
        "data", "html", "locale", "reportlab", "src", "tkRAD", "xml",
    )

    PYTHON = {
        "version": "3.3",
        "strict": False,
    }

    def finalize (self, *args, **kw):
        try:
            self.options.save()
            tkRAD.i18n.dump_pot_file()
        except:
            pass
        # end try
    # end def

    def initialize (self, *args, **kw):
        import os
        _cwd = os.path.realpath(
            os.path.join(os.path.dirname(__file__), "..")
        )
        kw.setdefault("app_root_dir", _cwd)
        os.chdir(_cwd)
        super().initialize(*args, **kw)
    # end def

    def main (self, *args, **kw):
        from tkinter import Tk
        self.root = Tk()
        self.root.withdraw()
        self.root.lift()
        from .splash_screen import SplashScreen
        _splash = SplashScreen(self.root, **self.APP)
        _splash.show()
        from .mainwindow import MainWindow
        self.mainwindow = MainWindow(self.root, **kw)
        self.mainwindow.show()
        _splash.wait_visibility(self.mainwindow)
        self.root.after(500, _splash.destroy)
        self.root.mainloop()
    # end def

# end class
