#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
    tkScenarist - screen writing made simpler

    Copyright (c) 2014+ Raphaël Seban <motus@laposte.net>

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.

    If not, see http://www.gnu.org/licenses/
"""

import tkRAD
from . import text_helpers as th


class ComponentPitchConcept (tkRAD.RADXMLFrame):

    def bind_events (self, **kw):
        self.events.connect_dict(
            {
                "Component:Reset": self.on_component_reset,
                "Pitch:Template:Insert": self.on_template_insert,
                "Reset:Modified": self.on_reset_modified,
            }
        )
        self.w_text.bind("<KeyRelease>", self.on_text_keypress)
    # end def

    def get_file_contents (self, files):
        import json
        return {
            files["text"]: th.text_get_contents(self.w_text),
            files["data"]: json.dumps({}),
        }
    # end def

    def initialize (self, *args, **kw):
        self.xml_build("component_pitch_concept")
        self.bind_events(**kw)
    # end def

    @property
    def modified (self):
        return self.w_text.edit_modified()
    # end def

    def on_component_reset (self, *args, **kw):
        th.text_reset_contents(self.w_text)
    # end def

    def on_reset_modified (self, *args, **kw):
        self.w_text.edit_modified(False)
    # end def

    def on_template_insert (self, *args, pitch=None, **kw):
        if pitch:
            th.text_set_contents(self.w_text, pitch)
        # end if
    # end def

    def on_text_keypress (self, event=None, *args, **kw):
        th.text_edit_autoseparator(self.w_text, event)
    # end def

    def setup_component (self, files, archive):
        import json
        _read = lambda f: archive.read(f).decode(ENCODING)
        _text = _read(files["text"])
        _data = json.loads(_read(files["data"]))
        th.text_reset_contents(self.w_text, _text)
    # end def

# end class
