#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
    tkScenarist - screen writing made simpler

    Copyright (c) 2014+ Raphaël Seban <motus@laposte.net>

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.

    If not, see http://www.gnu.org/licenses/
"""

import tkRAD
from . import text_helpers as th


class ComponentStoryboard (tkRAD.RADXMLFrame):

    def auto_save (self, *args, **kw):
        _index = self.listbox_shot.last_selected
        if _index >= 0:
            _shot = self.cvar_shot_nr.get()
            _title = self.entry_shot_title.get()
            _item = self.get_formatted_shot_text(_shot, _title)
            _text = th.text_get_contents(self.text_shot)
            _scene_nr, _shot_nr = _shot.strip("#").split(".")
            _scene_id = self.get_scene_id(_scene_nr)
            self.update_listbox_item(self.listbox_shot, _index, _item)
            self.database.stb_update_shot(
                scene=int(_scene_id),
                shot=int(_shot_nr),
                title=_title,
                text=_text,
            )
        # end if
    # end def

    def bind_events (self, **kw):
        self.events.connect_dict(
            {
                "Characters:List:Changed":
                    self.on_update_listbox_characters,

                "Component:Reset": self.on_component_reset,

                "Reset:Modified": self.on_reset_modified,

                "Scenario:Scene:Browser:Changed":
                    self.on_update_listbox_scene,

                "Storyboard:Shot:Add": self.on_shot_add,
                "Storyboard:Shot:Delete": self.on_shot_delete,
                "Storyboard:Shot:Purge": self.on_shot_purge,
            }
        )
        self.bind("<Expose>", self.on_expose)
        self.listbox_scene.bind(
            "<<ListboxSelect>>", self.on_listbox_select_scene
        )
        self.listbox_shot.bind(
            "<<ListboxSelect>>", self.on_listbox_select_shot
        )
        self.listbox_charnames.bind(
            "<<ListboxSelect>>", self.on_listbox_select_characters
        )
        self.text_shot.bind("<Key>", self.on_text_keypress)
        self.text_shot.bind("<ButtonRelease>", self.on_text_click)
        self.text_shot.bind("<MouseWheel>", self.on_text_click)
        self.text_shot.bind("<FocusIn>", self.on_text_click)
        self.text_shot.bind("<FocusOut>", self.on_text_focus_out)
        self.w_popup.bind("<FocusOut>", self.on_text_focus_out)
        self.w_popup.bind("<Button-1>", self.on_popup_click)
    # end def

    def clear_entries (self, *entries):
        for _entry in entries:
            self.enable_widget(_entry, True)
            _entry.delete(0, "end")
            self.enable_widget(_entry, False)
        # end for
    # end def

    def clear_listboxes (self, *listboxes):
        for _listbox in listboxes:
            _listbox.delete(0, "end")
            _listbox.selection_clear(0, "end")
            _listbox.last_selected = -1
        # end for
    # end def

    def clear_texts (self, *texts):
        for _text in texts:
            self.enable_widget(_text, True)
            th.text_reset_contents(_text)
            self.enable_widget(_text, False)
        # end for
    # end def

    def get_column_index (self, index=None):
        from tkRAD.core.tools import ensure_int
        index = self.text_shot.index(index or "insert")
        return ensure_int(index.split(".")[-1])
    # end def

    def get_component (self, component_name):
        return self.app.mainwindow.get_component(component_name)
    # end def

    def get_current_selected (self, listbox, force_index=-1):
        if 0 <= force_index < listbox.size():
            listbox.last_selected = force_index
        else:
            try:
                listbox.last_selected = listbox.curselection()[0]
            except:
                if not listbox.size():
                    self.clear_listboxes(listbox)
                # end if
            # end try
        # end if
        return listbox.last_selected
    # end def

    def get_file_contents (self, files):
        import json
        self.save_now()
        _rows = [dict(i) for i in self.database.stb_get_all_shots()]
        _fcontents = json.dumps(_rows)
        return {files: _fcontents}
    # end def

    def get_formatted_shot_text (self, shot_number, title=""):
        return "{} {}".format(shot_number, title)
    # end def

    def get_line_contents (self, index=None):
        index = index or "insert"
        return self.text_shot.get(
            "{} linestart".format(index), "{} lineend".format(index)
        )
    # end def

    def get_scene_id (self, scene_nr):
        from tkRAD.core.tools import ensure_int
        return self.listbox_scene.line_ids[ensure_int(scene_nr) - 1]
    # end def

    def get_scene_shot (self, index):
        _shot, _title = self.get_shot_chunks(
            self.listbox_shot.get(index)
        )
        _scene, _shot = _shot.strip("#").split(".")
        return (_scene, _shot)
    # end def

    def get_shot_chunks (self, text):
        if text:
            return text.split(" ", maxsplit=1)
        # end if
        return None
    # end def

    def get_shot_number (self, scene, shot):
        return "#{}.{:02d}".format(scene, shot)
    # end def

    def get_word (self, index=None):
        index = index or "insert"
        _start = "{} linestart".format(index)
        _end = "{} lineend".format(index)
        _word = ""
        _text = self.text_shot.get(_start, index)
        _pos = _text.rfind(" ")
        if _pos >= 0:
            _word += _text[_pos + 1:]
            _start = "{}+{}c".format(_start, _pos + 1)
        else:
            _word += _text
        # end if
        _text = self.text_shot.get(index, _end)
        _pos = _text.find(" ")
        if _pos >= 0:
            _word += _text[:_pos]
            _end = "{}+{}c".format(_start, _pos)
        else:
            _word += _text
        # end if
        return {
            "word": _word.rstrip(" .:,;?!\"']})"),
            "start_index": _start,
            "end_index": _end,
        }
    # end def

    def hide_popup_list (self, *args, **kw):
        self.async.stop(self.on_autocomplete)
        self.w_popup.hide()
    # end def

    def initialize (self, *args, **kw):
        from tkRAD.core.async import get_async_manager
        from .database import get_database
        from .text_completion_popup import TextCompletionPopup
        self.async = get_async_manager()
        self.database = get_database()
        self.xml_build("component_storyboard")
        self.listbox_scene.text_lines = []
        self.listbox_scene.line_ids = []
        self.listbox_shot.row_ids = []
        self.cvar_scene_nr = self.get_stringvar("lbl_scene_number")
        self.cvar_shot_nr = self.get_stringvar("lbl_shot_number")
        self.cvar_charname = self.get_stringvar("lbl_character_name")
        self.w_popup = TextCompletionPopup(
            self.text_shot, on_insert=self.on_popup_insert
        )
        self.clear_listboxes(
            self.listbox_scene, self.listbox_shot,
            self.listbox_charnames,
        )
        self.listbox_modified = False
        self.on_update_inputs()
        self.bind_events(**kw)
    # end def

    def listbox_delete (self, listbox, index):
        listbox.last_selected = -1
        listbox.delete(index)
        listbox.selection_clear(0, "end")
        index = max(-1, min(listbox.size() - 1, index))
        if index >= 0:
            listbox.see(index)
            listbox.selection_set(index)
            listbox.event_generate("<<ListboxSelect>>")
        # end if
        return index
    # end def

    @property
    def modified (self):
        return bool(
            self.text_shot.edit_modified() or self.listbox_modified
        )
    # end def

    def on_autocomplete (self, *args, **kw):
        _word = self.get_word("insert")
        _si = _word["start_index"]
        _names = (
            self.get_component("characters")
                .get_matching_names(_word["word"])
        )
        if not _names:
            _names = (
                self.get_component("characters")
                    .get_matching_names(
                        self.get_line_contents("insert")
                    )
            )
            _si = "insert linestart"
        # end if
        if _names:
            self.show_popup_list(choices=_names, start_index=_si)
        else:
            self.hide_popup_list()
        # end if
    # end def

    def on_component_reset (self, *args, **kw):
        self.database.stb_clear_shots()
        self.clear_listboxes(
            self.listbox_scene, self.listbox_shot,
            self.listbox_charnames
        )
        self.on_update_inputs()
    # end def

    def on_expose (self, *args, **kw):
        self.on_reset_selected_scene()
    # end def

    def on_listbox_select_characters (self, *args, **kw):
        _index = self.get_current_selected(self.listbox_charnames)
        if _index >= 0:
            _name = self.listbox_charnames.get(_index)
            self.cvar_charname.set(_name)
            self.enable_widget(self.text_characters_log, True)
            th.text_reset_contents(
                self.text_characters_log,
                self.get_component("characters")
                    .get_character_log(_name)
            )
            self.enable_widget(self.text_characters_log, False)
        # end if
    # end def

    def on_listbox_select_scene (self, *args, **kw):
        self.save_now()
        self.on_update_shot_listbox()
        self.on_update_scene_preview()
        self.on_update_inputs()
    # end def

    def on_listbox_select_shot (self, *args, **kw):
        self.save_now()
        try:
            _nb, _title = self.get_shot_chunks(
                self.listbox_shot.get(
                    self.get_current_selected(self.listbox_shot)
                )
            )
            _scene, _shot = _nb.strip("#").split(".")
            _scene_id = self.get_scene_id(_scene)
            _row = self.database.stb_get_shot(_scene_id, _shot)
            self.cvar_shot_nr.set(_nb)
            self.enable_widget(self.entry_shot_title, True)
            self.entry_shot_title.delete(0, "end")
            self.entry_shot_title.insert("end", _title)
            self.enable_widget(self.text_shot, True)
            th.text_reset_contents(self.text_shot, _row["text"])
            if not _title:
                self.after_idle(self.entry_shot_title.focus_set)
            else:
                self.after_idle(self.text_shot.focus_set)
            # end if
        except:
            pass
        # end try
        self.on_update_inputs()
    # end def

    def on_popup_click (self, *args, **kw):
        self.after_idle(
            self.async.stop, self.hide_popup_list, self.on_autocomplete
        )
    # end def

    def on_popup_insert (self, text, start_index):
        self.replace_text(text, start_index, smart_delete=True)
    # end def

    def on_reset_modified (self, *args, **kw):
        self.text_shot.edit_modified(False)
        self.listbox_modified = False
    # end def

    def on_reset_selected_scene (self, *args, force_index=None, **kw):
        _index = (
            force_index or
            self.get_current_selected(self.listbox_scene)
        )
        if _index >= 0:
            self.listbox_scene.last_selected = -1
            self.listbox_scene.see(_index)
            self.listbox_scene.selection_set(_index)
            self.on_listbox_select_scene()
        # end if
    # end def

    def on_shot_add (self, *args, **kw):
        _scene = self.get_current_selected(self.listbox_scene) + 1
        if _scene:
            try:
                _num, _title = self.get_shot_chunks(
                    self.listbox_shot.get("end")
                )
                _shot = int(_num.split(".")[-1]) + 1
            except:
                _shot = self.listbox_shot.size() + 1
            # end try
            self.listbox_shot.insert(
                "end",
                self.get_formatted_shot_text(
                    self.get_shot_number(_scene, _shot)
                )
            )
            self.listbox_shot.selection_clear(0, "end")
            self.listbox_shot.selection_set("end")
            self.listbox_shot.see("end")
            self.on_listbox_select_shot()
            self.listbox_modified = True
        # end if
    # end def

    def on_shot_delete (self, *args, **kw):
        _index = self.get_current_selected(self.listbox_shot)
        if _index >= 0:
            if not th.text_get_contents(self.text_shot) or \
                                        self.user_confirm_deletion():
                _scene, _shot = self.get_scene_shot(_index)
                _scene_id = self.get_scene_id(_scene)
                self.database.stb_del_shot(_scene_id, _shot)
                self.listbox_delete(self.listbox_shot, _index)
                self.listbox_modified = True
            # end if
        # end if
        self.on_update_inputs()
    # end def

    def on_shot_purge (self, *args, **kw):
        if self.user_confirm_purge():
            _scene_id = self.get_scene_id(
                self.get_current_selected(self.listbox_scene) + 1
            )
            self.database.stb_purge_shots(_scene_id)
            self.on_update_shot_listbox()
            self.listbox_modified = True
        # end if
        self.on_update_inputs()
    # end def

    def on_text_click (self, *args, **kw):
        self.hide_popup_list()
        if self.widget_enabled(self.text_shot):
            self.update_character_name()
        # end if
    # end def

    def on_text_focus_out (self, *args, **kw):
        self.async.run_after(100, self.hide_popup_list)
    # end def

    def on_text_keypress (self, event=None, *args, **kw):
        th.text_edit_autoseparator(self.text_shot, event)
        self.update_character_name()
        self.async.run_after(3000, self.auto_save)
    # end def

    def on_update_inputs (self, *args, **kw):
        _cur_scene = bool(
            self.get_current_selected(self.listbox_scene) + 1
        )
        _cur_shot = bool(
            self.get_current_selected(self.listbox_shot) + 1
        )
        _cur_char = bool(
            self.get_current_selected(self.listbox_charnames) + 1
        )
        self.enable_widget(self.btn_add_shot, _cur_scene)
        self.enable_widget(self.btn_del_shot, _cur_shot)
        self.enable_widget(
            self.btn_purge_shot, bool(self.listbox_shot.size())
        )
        if not _cur_scene:
            self.cvar_scene_nr.set("")
            self.clear_texts(self.text_scene_preview)
        # end if
        if _cur_shot:
            self.enable_widget(self.entry_shot_title, True)
            self.enable_widget(self.text_shot, True)
        else:
            self.cvar_shot_nr.set("")
            self.clear_entries(self.entry_shot_title)
            self.clear_texts(self.text_shot)
        # end if
        if not _cur_char:
            self.cvar_charname.set("")
            self.clear_texts(self.text_characters_log)
        # end if
    # end def

    def on_update_listbox_characters (self, *args, **kw):
        self.clear_listboxes(self.listbox_charnames)
        self.listbox_charnames.insert(
            "end", *(kw.get("contents") or [])
        )
    # end def

    def on_update_listbox_scene (self, *args, **kw):
        self.listbox_scene.text_lines = kw.get("lines") or []
        self.listbox_scene.line_ids = kw.get("line_ids") or []
        self.async.run_after_idle(
            self.database.stb_shots_cleanup,
            self.listbox_scene.line_ids
        )
        self.clear_listboxes(self.listbox_scene)
        self.listbox_scene.insert("end", *(kw.get("contents") or []))
        self.get_current_selected(
            self.listbox_scene, force_index=kw.get("current_selected")
        )
        self.on_listbox_select_scene()
        if self.winfo_ismapped():
            self.listbox_scene.selection_set(
                self.get_current_selected(self.listbox_scene)
            )
        # end if
    # end def

    def on_update_scene_preview (self, *args, **kw):
        _index = self.get_current_selected(self.listbox_scene)
        if _index >= 0:
            self.cvar_scene_nr.set("#{}".format(_index + 1))
            _start = float(self.listbox_scene.text_lines[_index])
            try:
                _end = "{}.0".format(
                    self.listbox_scene.text_lines[_index + 1]
                )
            except:
                _end = "end"
            # end try
            _scenario = self.get_component("scenario").w_text
            self.enable_widget(self.text_scene_preview, True)
            th.text_reset_contents(self.text_scene_preview)
            self.text_scene_preview.insert(
                "end", *_scenario.get_tagged_text(_start, _end)
            )
            self.enable_widget(self.text_scene_preview, False)
            _scenario.copy_styles_into(self.text_scene_preview)
        # end if
    # end def

    def on_update_shot_listbox (self, *args, **kw):
        _scene = self.get_current_selected(self.listbox_scene) + 1
        if _scene:
            self.clear_listboxes(self.listbox_shot)
            _scene_id = self.get_scene_id(_scene)
            for _row in self.database.stb_get_shot_list(_scene_id):
                self.listbox_shot.insert(
                    "end",
                    self.get_formatted_shot_text(
                        self.get_shot_number(_scene, _row["shot"]),
                        _row["title"]
                    )
                )
            # end for
        # end if
    # end def

    def replace_text (self, text, start=None, end=None,
                                smart_delete=False, keep_cursor=False):
        start = start or "insert"
        end = end or "insert"
        _cursor = self.text_shot.index("insert")
        if smart_delete:
            import re
            _endl = "{} lineend".format(end)
            _text = self.text_shot.get(end, _endl)
            _found = re.search(r"[^\w\-]", _text)
            if _found:
                end = "{} +{}c".format(end, _found.start())
            else:
                end = _endl
            # end if
        # end if
        self.text_shot.delete(start, end)
        self.text_shot.insert(start, text)
        if keep_cursor:
            self.text_shot.mark_set("insert", _cursor)
        # end if
    # end def

    def save_now (self):
        self.async.stop(self.auto_save)
        self.auto_save()
    # end def

    def setup_component (self, files, archive):
        import json
        _rows = json.loads(files or "[]")
        self.database.stb_import_shots(_rows)
    # end def

    def show_popup_list (self, *args, **kw):
        self.async.stop(self.hide_popup_list, self.on_autocomplete)
        self.w_popup.show(*args, **kw)
    # end def

    def update_character_name (self, *args, **kw):
        _line = self.get_line_contents("insert")
        _characters = self.get_component("characters")
        _name, _start_index = _characters.find_nearest_name(
            _line, self.get_column_index("insert")
        )
        if _characters.is_registered(_name):
            if _name not in _line:
                _index = "insert linestart+{}c"
                self.replace_text(
                    _name,
                    _index.format(_start_index),
                    _index.format(_start_index + len(_name)),
                    keep_cursor=True,
                )
            # end if
            self.hide_popup_list()
        else:
            self.async.run_after_idle(self.on_autocomplete)
        # end if
    # end def

    def update_listbox_item (self, listbox, index, text):
        _sel = listbox.curselection()
        listbox.delete(index)
        listbox.insert(index, text)
        if _sel:
            listbox.see(_sel[0])
            listbox.selection_set(_sel[0])
        # end if
    # end def

    def user_confirm_deletion (self):
        from tkinter.messagebox import askyesno
        return askyesno(
            title=_("Attention"),
            message=_("Do you really want to delete selected shot?"),
            parent=self
        )
    # end def

    def user_confirm_purge (self):
        from tkinter.messagebox import askyesno
        return askyesno(
            title=_("Attention"),
            message=_(
                "Do you really want to remove *ALL* empty shots "
                "from list?"
            ),
            parent=self
        )
    # end def

# end class
