#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
    tkScenarist - screen writing made simpler

    Copyright (c) 2014+ Raphaël Seban <motus@laposte.net>

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.

    If not, see http://www.gnu.org/licenses/
"""

import tkRAD


class ComponentTitleData (tkRAD.RADXMLFrame):

    FIELD_CVARNAMES = (
        "project_title",
        "project_subtitle",
        "project_episode",
        "project_author",
        "project_author_email",
        "project_author_phone",
    )

    INFO_CVARNAMES = (
        "project_filename",
        "project_directory",
        "scenario_nb_of_pages",
        "scenario_movie_duration",
    )

    def bind_events (self, *args, **kw):
        self.events.connect_dict(
            {
                "Component:Reset": self.on_component_reset,
                "Project:Path:Update": self.on_path_update,
                "Reset:Modified": self.on_reset_modified,
                "Scenario:Stats:Updated": self.on_stats_updated,
            }
        )
        for _cvarname in self.FIELD_CVARNAMES:
            self.get_stringvar(_cvarname).trace(
                "w", self.on_entry_modified
            )
        # end for
        self.bind_class("TEntry", "<Expose>", self.on_expose_entry)
    # end def

    def cvar_set_text (self, cvarname, contents):
        self.get_stringvar(cvarname).set(contents)
    # end def

    def get_data (self):
        return {
            _cvarname: self.get_stringvar(_cvarname).get()
            for _cvarname in self.FIELD_CVARNAMES
        }
    # end def

    def get_file_contents (self, files):
        import json
        _fcontents = json.dumps(self.get_data())
        return {files: _fcontents}
    # end def

    def initialize (self, *args, **kw):
        self.xml_build("component_title_data")
        self.bind_events(*args, **kw)
        self.modified = False
    # end def

    @property
    def modified (self):
        return self.__modified
    # end def

    @modified.setter
    def modified (self, value):
        self.__modified = bool(value)
    # end def

    def on_component_reset (self, *args, **kw):
        for _cvar in self.get_stringvars().values():
            _cvar.set("")
        # end for
    # end def

    def on_entry_modified (self, *args, **kw):
        self.modified = True
    # end def

    def on_expose_entry (self, event=None, *args, **kw):
        try:
            event.widget.selection_clear()
        except:
            pass
        # end try
    # end def

    def on_path_update (self, *args, filename=None, directory=None,
                                                                **kw):
        if filename and directory:
            self.cvar_set_text("project_filename", filename)
            self.cvar_set_text("project_directory", directory)
        # end if
    # end def

    def on_reset_modified (self, *args, **kw):
        self.modified = False
    # end def

    def on_stats_updated (self, *args, **kw):
        self.cvar_set_text(
            "scenario_nb_of_pages", kw.get("total_pages") or ""
        )
        self.cvar_set_text(
            "scenario_movie_duration",
            kw.get("movie_duration_label") or ""
        )
    # end def

    def setup_component (self, files, archive):
        try:
            import json
            _data = json.loads(files)
            for _cvarname, _value in _data.items():
                self.cvar_set_text(_cvarname, _value)
            # end for
        except:
            # FIXME: remove this in future versions e.g. in v2.0
            # backward compatibility
            for _line in files.split("\n"):
                try:
                    _name, _value = _line.split(":", maxsplit=1)
                    self.cvar_set_text(
                        _name.strip().lower(), _value.strip()
                    )
                except:
                    pass
                # end try
            # end for
        # end try
    # end def

# end class
