#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
    tkScenarist - screen writing made simpler

    Copyright (c) 2014+ Raphaël Seban <motus@laposte.net>

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.

    If not, see http://www.gnu.org/licenses/
"""

from reportlab.platypus import Paragraph


def build_document (doc):
    doc.build_document()
    return doc.progress
# end def


def build_elements (doc):
    doc.build_elements()
    return doc.progress
# end def


def get_pdf_document (doc_name, **kw):
    return eval(
        "PDFDocument{name}(doc_name, **kw)"
        .format(name=str(doc_name).title().replace("_", ""))
    )
# end def


def get_stylesheet ():
    from reportlab.lib.styles import ParagraphStyle
    from reportlab.lib.units import inch
    from reportlab.lib.enums import (
        TA_LEFT, TA_CENTER, TA_RIGHT, TA_JUSTIFY
    )
    _root_style = ParagraphStyle(
        "root_style",
        fontName="Courier",
        fontSize=10,
        leading=12,
        alignment=TA_LEFT,
        leftIndent=0,
        rightIndent=0,
        spaceBefore=0,
        spaceAfter=0,
    )
    return {
        "header": ParagraphStyle(
            "header",
            parent=_root_style,
            fontName="Times-BoldItalic",
            fontSize=12,
            leading=12,
            alignment=TA_CENTER,
        ),
        "h1": ParagraphStyle(
            "h1",
            parent=_root_style,
            fontName="Times-Bold",
            fontSize=24,
            leading=26,
            underlineProportion=1,
            alignment=TA_CENTER,
            spaceAfter=0.3*inch,
        ),
        "h2": ParagraphStyle(
            "h2",
            parent=_root_style,
            fontName="Times-Bold",
            fontSize=14,
            leading=16,
            alignment=TA_LEFT,
            spaceAfter=0.1*inch,
        ),
        "h3": ParagraphStyle(
            "h3",
            parent=_root_style,
            fontName="Times-Bold",
            fontSize=12,
            leading=14,
            alignment=TA_LEFT,
            spaceAfter=0.1*inch,
        ),
        "body": ParagraphStyle(
            "body",
            parent=_root_style,
            fontName="Helvetica",
            alignment=TA_JUSTIFY,
            spaceAfter=0.1*inch,
            bulletIndent=0.1*inch,
        ),
        "footer": ParagraphStyle(
            "footer",
            parent=_root_style,
            fontName="Times-Italic",
            alignment=TA_CENTER,
        ),
        "footer_tiny": ParagraphStyle(
            "footer_tiny",
            parent=_root_style,
            fontName="Helvetica-Oblique",
            fontSize=6,
            leading=6,
            alignment=TA_CENTER,
        ),
        "th": ParagraphStyle(
            "th",
            parent=_root_style,
            fontName="Times-Bold",
            fontSize=12,
            leading=14,
            alignment=TA_CENTER,
        ),
        "td": ParagraphStyle(
            "td",
            parent=_root_style,
            fontName="Helvetica",
            alignment=TA_JUSTIFY,
            spaceAfter=0.1*inch,
        ),


        "title": ParagraphStyle(
            "title",
            fontName="Times-Bold",
            fontSize=36,
            leading=40,
            alignment=TA_CENTER,
            leftIndent=0.5*inch,
            rightIndent=0.5*inch,
            spaceBefore=0,
            spaceAfter=0.1*inch,
        ),
        "subtitle": ParagraphStyle(
            "subtitle",
            fontName="Times-Italic",
            fontSize=18,
            leading=20,
            alignment=TA_CENTER,
            leftIndent=1*inch,
            rightIndent=1*inch,
            spaceBefore=0,
            spaceAfter=0.1*inch,
        ),
        "episode": ParagraphStyle(
            "episode",
            fontName="Helvetica",
            fontSize=10,
            leading=12,
            alignment=TA_CENTER,
            leftIndent=1.5*inch,
            rightIndent=1.5*inch,
            spaceBefore=0,
            spaceAfter=0.1*inch,
        ),
        "author": ParagraphStyle(
            "author",
            fontName="Times-BoldItalic",
            fontSize=16,
            leading=18,
            alignment=TA_CENTER,
            leftIndent=1.5*inch,
            rightIndent=1.5*inch,
            spaceBefore=0.3*inch,
            spaceAfter=0.1*inch,
        ),
        "contact": ParagraphStyle(
            "contact",
            fontName="Times-Italic",
            fontSize=12,
            leading=12,
            alignment=TA_LEFT,
            leftIndent=0,
            rightIndent=0,
            spaceBefore=0,
            spaceAfter=6,
        ),


        "charname": ParagraphStyle(
            "charname",
            parent=_root_style,
            fontName="Helvetica-Bold",
            fontSize=14,
            leading=16,
            alignment=TA_LEFT,
            spaceBefore=0.3*inch,
            spaceAfter=0.1*inch,
            keepWithNext=True,
        ),
        "charlog": ParagraphStyle(
            "charlog",
            parent=_root_style,
            fontName="Helvetica",
            alignment=TA_JUSTIFY,
            leftIndent=0.1*inch,
            rightIndent=0.1*inch,
            spaceAfter=0.1*inch,
        ),


        "action": ParagraphStyle(
            "action",
            parent=_root_style,
            alignment=TA_JUSTIFY,
            spaceAfter=0.1*inch,
        ),
        "character": ParagraphStyle(
            "character",
            parent=_root_style,
            fontName="Courier-Bold",
            alignment=TA_CENTER,
            leftIndent=2*inch,
            rightIndent=2*inch,
            keepWithNext=True,
        ),
        "dialogue": ParagraphStyle(
            "dialogue",
            parent=_root_style,
            alignment=TA_JUSTIFY,
            leftIndent=1*inch,
            rightIndent=1*inch,
            spaceAfter=0.1*inch,
        ),
        "parenthetical": ParagraphStyle(
            "parenthetical",
            parent=_root_style,
            fontName="Courier-Oblique",
            alignment=TA_CENTER,
            leftIndent=1.5*inch,
            rightIndent=1.5*inch,
            keepWithNext=True,
        ),
        "scene": ParagraphStyle(
            "scene",
            parent=_root_style,
            fontName="Courier-Bold",
            backColor="#f0f0f0",
            spaceAfter=0.1*inch,
            keepWithNext=True,
        ),
        "transition": ParagraphStyle(
            "transition",
            parent=_root_style,
            alignment=TA_RIGHT,
            spaceAfter=0.1*inch,
        ),
        "stats": ParagraphStyle(
            "stats",
            parent=_root_style,
            fontName="Times",
            alignment=TA_LEFT,
            leftIndent=0.25*inch,
            spaceAfter=0.1*inch,
        ),


        "shot_title": ParagraphStyle(
            "shot_title",
            parent=_root_style,
            fontName="Courier-Bold",
            backColor="#f0f0f0",
            spaceAfter=0.1*inch,
            keepWithNext=True,
        ),
        "shot_body": ParagraphStyle(
            "shot_body",
            parent=_root_style,
            alignment=TA_JUSTIFY,
            leftIndent=0.2*inch,
            rightIndent=0.2*inch,
            spaceAfter=0.1*inch,
        ),
    }
# end def


class PDFDocumentBase:

    FRAME_OPTIONS = {
        "leftPadding": 0,
        "rightPadding": 0,
        "topPadding": 0,
        "bottomPadding": 0,
        "showBoundary": 0,
    }

    def __init__ (self, doc_name, **kw):
        from tkRAD.core.services import ask_for
        from .database import get_database
        self.app = ask_for("app") # application
        self.pfm = ask_for("PFM") # Project File Management
        self.database = get_database()
        self.filepath = None
        self.doc_name = doc_name
        self.options = kw.get("options")
        self.project_data = kw.get("data")
        self.document = self.get_pdf_document(doc_name)
        self.styles = get_stylesheet()
        self.elements = []
        self.reset_progress(force_reset=True)
    # end def

    def _thread_build (self):
        self.document.build(
            self.elements,
            onFirstPage=self.draw_first_page,
            onLaterPages=self.draw_pages,
        )
    # end def

    def add_pagebreak (self):
        from reportlab.platypus import PageBreak
        self.elements.append(PageBreak())
    # end def

    def add_paragraph (self, text, style, bulletText=None):
        if text and isinstance(text, str):
            self.elements.append(Paragraph(text, style, bulletText))
        # end if
    # end def

    def build_document (self):
        self.reset_progress()
        if not self.step:
            import threading
            self.step = 1
            self.index = 4700.0 / (1.0 + len(self.elements))
            self._thread = threading.Thread(target=self._thread_build)
            self._thread.start()
        else:
            self.progress = min(99.0, self.progress + self.index)
            if not self._thread.is_alive():
                self.progress = 100
                del self._thread
            # end if
        # end if
    # end def

    def build_elements (self):
        self.reset_progress()
        self.set_first_page_elements()
        for i in range(20):
            self.add_paragraph("*** TEST ***", self.styles["body"])
        # end if
        self.progress = 100
    # end def

    def draw_first_page (self, canvas, doc):
        from datetime import datetime
        from reportlab.platypus import Frame
        from reportlab.platypus.frames import ShowBoundaryValue
        canvas.saveState()
        SeqNumberParagraph.reset()
        _data = self.project_data
        _width, _height = doc.pagesize
        _margin_w = _width - doc.leftMargin - doc.rightMargin
        _style = self.styles["header"]
        _frame_h = _style.fontSize + 4
        Frame(
            doc.leftMargin, _height - 0.75 * doc.topMargin,
            _margin_w, _frame_h, **self.FRAME_OPTIONS
        ).addFromList([Paragraph(self.fancy_name, _style)], canvas)
        _style = self.styles["contact"]
        _frame_h = _style.fontSize + _style.spaceAfter
        Frame(
            doc.leftMargin, doc.bottomMargin,
            _margin_w, _frame_h * 4 - _style.spaceAfter,
            showBoundary=ShowBoundaryValue(),
        ).addFromList(
            [
                Paragraph(_t.format(**_data), _style)
                for _t in (
                    _("Contact: {project_author}"),
                    _("E-mail: {project_author_email}"),
                    _("Phone: {project_author_phone}"),
                )
            ],
            canvas
        )
        _footer = self.styles["footer"]
        _mentions = self.styles["footer_tiny"]
        _frame_h = _footer.fontSize + _mentions.fontSize + 8
        Frame(
            doc.leftMargin, 0.5 * doc.bottomMargin,
            _margin_w, _frame_h, **self.FRAME_OPTIONS
        ).addFromList(
            [
                Paragraph(
                    _("Printed on {date}")
                    .format(date=datetime.today().strftime("%c")),
                    _footer
                ),
                Paragraph(
                    _(
                        "Document generated by {name} with "
                        "{pdflib} PDF toolkit"
                    ).format(**self.app.APP),
                    _mentions
                ),
            ],
            canvas
        )
        canvas.restoreState()
    # end def

    def draw_pages (self, canvas, doc):
        from reportlab.platypus import Frame
        canvas.saveState()
        _data = self.project_data
        _width, _height = doc.pagesize
        _margin_w = _width - doc.leftMargin - doc.rightMargin
        _text = _data["project_title"].strip()
        _text = "{}{} - {}".format(
            _text[:60].strip(),
            len(_text) >= 60 and "..." or "",
            self.fancy_name
        )
        _style = self.styles["header"]
        _frame_h = _style.fontSize + 4
        Frame(
            doc.leftMargin, _height - 0.75 * doc.topMargin,
            _margin_w, _frame_h, **self.FRAME_OPTIONS
        ).addFromList([Paragraph(_text, _style)], canvas)
        _text = _("Page {number}").format(number=doc.page)
        _style = self.styles["footer"]
        _frame_h = _style.fontSize + 4
        Frame(
            doc.leftMargin, 0.5 * doc.bottomMargin,
            _margin_w, _frame_h, **self.FRAME_OPTIONS
        ).addFromList([Paragraph(_text, _style)], canvas)
        canvas.restoreState()
    # end def

    @property
    def fancy_name (self):
        return _(str(self.doc_name).title().replace("_", "/"))
    # end def

    def get_component (self, component_name):
        return self.app.mainwindow.get_component(component_name)
    # end def

    def get_pdf_document (self, doc_name):
        if doc_name and isinstance(doc_name, str):
            from os.path import splitext
            from reportlab.platypus import SimpleDocTemplate
            from tkRAD.core.path import normalize
            _data = self.project_data
            _fpath, _fext = splitext(self.pfm.project_path)
            self.filepath = normalize(
                "{}-{}.pdf".format(_fpath, doc_name)
            )
            return SimpleDocTemplate(
                filename=self.filepath,
                author=_data.get("project_author"),
                title="{project_title} - "
                "{project_subtitle} - "
                "{project_episode}"
                .format(**_data),
                subject=self.fancy_name,
                creator=self.app.APP.get("title"),
                showBoundary=0,
            )
        else:
            raise ValueError(
                "parameter 'doc_name' must be of plain string type."
            )
        # end if
    # end def

    @property
    def progress (self):
        return self.__progress
    # end def

    @progress.setter
    def progress (self, value):
        self.__progress = float(min(100, max(0, value)))
    # end def

    @progress.deleter
    def progress (self):
        del self.__progress
    # end def

    def reset_progress (self, force_reset=False):
        if force_reset or self.progress >= 100:
            self.progress = 0
            self.index = 0
            self.step = 0
            self.read_bytes = 0
            self.total_bytes = 0
        # end if
    # end def

    def set_first_page_elements (self):
        from reportlab.platypus import Spacer
        from reportlab.lib.units import inch
        from tkRAD.core import tools
        _data = self.project_data
        _styles = self.styles
        self.elements.clear()
        self.elements.append(Spacer(0, 2*inch))
        self.add_paragraph(
            _data.get("project_title"), _styles["title"]
        )
        self.add_paragraph(
            _data.get("project_subtitle"), _styles["subtitle"]
        )
        self.add_paragraph(
            _data.get("project_episode"), _styles["episode"]
        )
        self.add_paragraph(
            tools.str_complete(
                _("By {}"), _data.get("project_author")
            ),
            _styles["author"]
        )
        self.add_pagebreak()
    # end def

# end class


class PDFDocumentBaseText (PDFDocumentBase):

    def __init__ (self, doc_name, **kw):
        super().__init__(doc_name, **kw)
        self.init_document(**kw)
    # end def

    def build_elements (self):
        self.reset_progress()
        if not self.step:
            self.init_elements()
        else:
            self.do_steps()
        # end if
    # end def

    def do_steps (self):
        _text = self.w_text.get(self.index, self.index + 100.0)
        self.index += 100.0
        self.read_bytes += len(_text)
        self.progress = min(
            99.0, 100.0 * self.read_bytes / self.total_bytes
        )
        if not _text:
            self.progress = 100
        else:
            for _paragraph in _text.split("\n"):
                self.add_paragraph(_paragraph, self.styles["body"])
            # end for
        # end if
    # end def

    def estimate_text_size (self):
        _paragraphs = float(self.w_text.index("end"))
        if _paragraphs < 3000:
            self.total_bytes = len(self.w_text.get(1.0, "end"))
        else:
            self.total_bytes = _paragraphs * 360
        # end if
        self.total_bytes = max(1, self.total_bytes)
    # end def

    def init_document (self, **kw):
        self.w_text = None
    # end def

    def init_elements (self):
        self.reset_progress(force_reset=True)
        self.step = 1
        self.index = 1.0
        self.estimate_text_size()
        self.set_first_page_elements()
    # end def

# end class


class PDFDocumentCharacters (PDFDocumentBase):

    def __init__ (self, doc_name, **kw):
        super().__init__(doc_name, **kw)
        _characters = self.get_component("characters")
        _canvas = _characters.w_canvas
        self.character_logs = _characters.character_logs
        self.character_labels = _canvas.character_labels
        self.relationships = _canvas.get_named_relationships()
    # end def

    def build_elements (self):
        self.reset_progress()
        exec("self.do_step_{}()".format(self.step))
    # end def

    def do_step_0 (self):
        self.reset_progress(force_reset=True)
        self.sorted_names = sorted(self.character_logs)
        self.index = 0
        self.set_first_page_elements()
        self.step += 1
    # end def

    def do_step_1 (self):
        if self.character_logs:
            self.add_paragraph(_("Statistics"), self.styles["h2"])
            self.add_paragraph(
                _("Known character names: {name_count}")
                .format(name_count=len(self.character_logs)),
                self.styles["body"]
            )
            self.step += 1
        else:
            self.progress = 100
        # end if
    # end def

    def do_step_2 (self):
        try:
            _name = self.sorted_names[self.index]
            self.add_paragraph(_name, self.styles["charname"])
            _log = self.character_logs[_name].strip()
            if _log:
                for _p in _log.split("\n"):
                    self.add_paragraph(_p, self.styles["charlog"])
                # end for
            else:
                self.add_paragraph(
                    "<i>{}</i>".format(_("(no history log)")),
                    self.styles["charlog"]
                )
            # end if
            self.progress = 50.0 * self.index / len(self.sorted_names)
            self.index += 1
        # end of list
        except:
            self.progress = 50
            self.step += 1
        # end try
    # end def

    def do_step_3 (self):
        if not self.relationships:
            self.progress = 100
            return
        # end if
        self.add_pagebreak()
        self.add_paragraph(_("Relationships"), self.styles["h1"])
        _count = len(self.relationships)
        self.add_paragraph(_("Statistics"), self.styles["h2"])
        self.add_paragraph(
            _("Known distinct mutual relationships: {dmr}")
            .format(dmr=_count),
            self.styles["body"]
        )
        self.add_paragraph(
            _("Listed relationships: {list_count}")
            .format(list_count=_count*2),
            self.styles["body"]
        )
        self.add_paragraph(_("Relationships"), self.styles["h2"])
        self.table_rows = []
        self.table_rows.append(
            (
                Paragraph(_("Name"), self.styles["th"]),
                Paragraph(_("Name"), self.styles["th"]),
                Paragraph(_("Relationship"), self.styles["th"]),
            )
        )
        self.sorted_names = sorted(self.character_labels)
        self.index = 0
        self.step += 1
    # end def

    def do_step_4 (self):
        try:
            _td = self.styles["td"]
            _name = self.sorted_names[self.index]
            _relationships = self.get_relationships_for(_name)
            for _name2 in sorted(_relationships):
                self.table_rows.append(
                    (
                        Paragraph(_name, _td),
                        Paragraph(_name2, _td),
                        Paragraph(_relationships[_name2], _td),
                    )
                )
            # end for
            self.progress = min(
                99.0,
                50.0 * (1.0 + self.index / len(self.sorted_names))
            )
            self.index += 1
        # end of list
        except:
            from reportlab.lib import colors
            from reportlab.platypus import Table
            _style = [
                ("BOX", (0, 0), (-1, -1), 0.2, colors.black),
                ("INNERGRID", (0, 0), (-1, -1), 0.1, colors.black),
                ("VALIGN", (0, 0), (-1, -1), "MIDDLE"),
            ]
            self.elements.append(
                Table(self.table_rows, repeatRows=1, style=_style)
            )
            self.progress = 100
        # end try
    # end def

    def get_relationships_for (self, character_name):
        _dict = dict()
        for _group in self.relationships:
            if _group["name0"] == character_name:
                _dict[_group["name1"]] = _group["text"]
            elif _group["name1"] == character_name:
                _dict[_group["name0"]] = _group["text"]
            # end if
        # end for
        return _dict
    # end def

# end class


class PDFDocumentDraftNotes (PDFDocumentBaseText):

    def __init__ (self, doc_name, **kw):
        super().__init__(doc_name, **kw)
        self.draft_notes = (
            self.get_component("draft_notes").draft_notes.copy()
        )
    # end def

    def build_elements (self):
        self.reset_progress()
        exec("self.do_step_{}()".format(self.step))
    # end def

    def do_step_0 (self):
        self.reset_progress(force_reset=True)
        self.sorted_notes = sorted(self.draft_notes)
        self.total_notes = len(self.draft_notes)
        self.index = 0
        self.set_first_page_elements()
        self.step += 1
    # end def

    def do_step_1 (self):
        try:
            _title = self.sorted_notes.pop(0)
        except:
            self.progress = 100
        else:
            self.progress = min(
                99.0, 100.0 * self.index / self.total_notes
            )
            self.index += 1
            self.text = self.draft_notes.pop(_title, "")
            if self.text:
                self.read_bytes = 0
                self.total_bytes = len(self.text)
                self.add_paragraph(_title, self.styles["h1"])
                self.text = str(self.text).split("\n")
                self.step += 1
            # end if
        # end try
    # end def

    def do_step_2 (self):
        try:
            _ratio = (self.index - 1) / self.total_notes
            _total = self.total_bytes * self.total_notes
            for i in range(10):
                _paragraph = self.text.pop(0)
                self.add_paragraph(_paragraph, self.styles["body"])
                self.read_bytes += len(_paragraph)
                self.progress = min(
                    99.0, 100.0 * (_ratio + self.read_bytes / _total)
                )
            # end for
        except:
            self.step -= 1
        # end try
    # end def

# end class


class PDFDocumentPitchConcept (PDFDocumentBaseText):

    def init_document (self, **kw):
        self.w_text = self.get_component("pitch_concept").w_text
    # end def

# end class


class PDFDocumentResources (PDFDocumentBase):

    ITEM_FIELDNAMES = (
        "name", "role", "contact", "phone", "email", "notes"
    )

    def add_item_data (self, fk_type):
        _row = self.database.res_get_item(fk_type)
        if _row:
            for _key in self.ITEM_FIELDNAMES:
                _value = _row[_key]
                if _value:
                    self.add_paragraph(
                        _("{key}: {value}")
                        .format(
                            key=_(_key.capitalize()), value=_value
                        ),
                        self.styles["body"],
                        bulletText="\u2022",
                    )
                # end if
            # end for
        else:
            self.notify_no_data(0)
        # end if
    # end def

    def add_item_datebars (self, fk_type):
        _rows = self.database.res_get_datebars([fk_type])
        if _rows:
            _table_rows = [
                (
                    Paragraph(_("Date begin"), self.styles["th"]),
                    Paragraph(_("Date end"), self.styles["th"]),
                    Paragraph(_("Availability"), self.styles["th"]),
                ),
            ]
            _td = self.styles["td"]
            for _row in _rows:
                _table_rows.append(
                    (
                        Paragraph(
                            self.convert_iso_date(_row["date_begin"]),
                            _td
                        ),
                        Paragraph(
                            self.convert_iso_date(_row["date_end"]),
                            _td
                        ),
                        Paragraph(
                            _("Available")
                            if _row["status"].upper() == "OK"
                            else _("NOT available"),
                            _td
                        ),
                    )
                )
            # end for
            from reportlab.lib import colors
            from reportlab.platypus import Table
            from reportlab.lib.units import inch
            _style = [
                ("BOX", (0, 0), (-1, -1), 0.2, colors.black),
                ("INNERGRID", (0, 0), (-1, -1), 0.1, colors.black),
                ("VALIGN", (0, 0), (-1, -1), "MIDDLE"),
            ]
            _table = Table(_table_rows, repeatRows=1, style=_style)
            _table.spaceBefore = 0.2*inch
            _table.spaceAfter = 0.2*inch
            self.elements.append(_table)
        # end if
    # end def

    def build_elements (self):
        self.reset_progress()
        exec("self.do_step_{}()".format(self.step))
    # end def

    def convert_iso_date (self, date_str):
        from datetime import datetime
        return (
            datetime.strptime(date_str, "%Y-%m-%d").strftime("%A %x")
        )
    # end def

    def do_step_0 (self):
        self.reset_progress(force_reset=True)
        self.sections = self.database.res_get_types()
        self.sorted_sections = sorted(self.sections)
        self.total_sections = len(self.sections)
        self.index = 0
        self.set_first_page_elements()
        self.step += 1
    # end def

    def do_step_1 (self):
        try:
            _section = self.sorted_sections.pop(0)
        except:
            self.notify_no_data(self.total_sections)
            self.progress = 100
        else:
            self.progress = min(
                99.0, 100.0 * self.index / self.total_sections
            )
            self.index += 1
            self.add_paragraph(_section, self.styles["h1"])
            self.subsections = self.database.res_get_types(
                self.sections[_section]
            )
            self.sorted_subsections = sorted(self.subsections)
            self.total_subsections = len(self.subsections)
            self.step += 1
        # end try
    # end def

    def do_step_2 (self):
        try:
            _subsection = self.sorted_subsections.pop(0)
        except:
            self.notify_no_data(self.total_subsections)
            self.add_pagebreak()
            self.step = 1
        else:
            self.progress = min(
                99.0,
                self.progress +
                100.0 / (self.total_sections * self.total_subsections)
            )
            self.add_paragraph(_subsection, self.styles["h2"])
            self.items = self.database.res_get_types(
                self.subsections[_subsection]
            )
            self.sorted_items = sorted(self.items)
            self.total_items = len(self.items)
            self.step += 1
        # end try
    # end def

    def do_step_3 (self):
        try:
            _item = self.sorted_items.pop(0)
        except:
            self.notify_no_data(self.total_items)
            self.step = 2
        else:
            _key = self.items[_item]
            self.add_paragraph(_item, self.styles["h3"])
            self.add_item_data(_key)
            self.add_item_datebars(_key)
        # end try
    # end def

    def notify_no_data (self, count):
        if not count:
            self.add_paragraph(
                "<i>{}</i>".format(_("(no data)")),
                self.styles["body"]
            )
        # end if
    # end def

# end class


class PDFDocumentScenario (PDFDocumentBaseText):

    def add_stats_elements (self):
        _s = self.stats
        _ds = self.styles["stats"]
        _elements = (
            (_("Statistics"), "h1"),
            (_("Document"), "h2"),
            (PageNumber(_("Pages: {field}"), _ds), "*"),
            (_("Paragraphs: {paragraph_count}").format(**_s), ""),
            (_("Words: {word_count}").format(**_s), ""),
            (_("Glyphs (letters/signs): {byte_count}").format(**_s),
                ""),
            (_("Scenario"), "h2"),
            (PageNumber(_("Pages: {field}"), _ds, adjust=-2), "*"),
            (_("Scenes: {scene_count}").format(**_s), ""),
            (_("Dialogues: {dialogue_count}").format(**_s), ""),
            (_("Transitions: {transition_count}").format(**_s), ""),
            (_("Movie"), "h2"),
            (
                _("Movie protagonists: {char_count}")
                .format(char_count=len(_s["movie_chars"])),
                ""
            ),
            (
                PageNumber(
                    _("Estimated movie duration: {field}"),
                    _ds, adjust=-2, formatter=self.get_duration,
                ),
                "*"
            ),
            #~ (_("").format(), ""),
        )
        self.add_pagebreak()
        for (_p, _s) in _elements:
            if _s == "*":
                self.elements.append(_p)
            else:
                self.add_paragraph(_p, self.styles[_s or "stats"])
            # end if
        # end for
    # end def

    def do_steps (self):
        _tagged_text = self.w_text.get_tagged_text(
            self.index, self.index + 100.0
        )
        _only_texts = _tagged_text[::2]
        _only_tags = _tagged_text[1::2]
        self.index += 100.0
        _bytes = sum(map(len, _only_texts))
        self.read_bytes += _bytes
        self.progress = min(
            99.0, 100.0 * self.read_bytes / self.total_bytes
        )
        if not _bytes:
            self.add_stats_elements()
            self.progress = 100
        else:
            _print_left = self.options.get("print_scene_left")
            _print_right = self.options.get("print_scene_right")
            for _index, _text in enumerate(_only_texts):
                _text = _text.strip()
                try:
                    _style = self.styles[_only_tags[_index][0]]
                except:
                    _style = self.styles["body"]
                # end try
                for _line in _text.split("\n"):
                    _line = _line.strip()
                    if _line:
                        if _style.name == "scene":
                            self.elements.append(
                                SeqNumberParagraph(
                                    _line, _style,
                                    print_left=_print_left,
                                    print_right=_print_right,
                                )
                            )
                        else:
                            self.add_paragraph(_line, _style)
                        # end if
                    # end if
                    self.update_stats(_line, _style.name)
                # end for
            # end for
        # end if
    # end def

    def get_duration (self, duration):
        return _(
            "{hours:02d} h {minutes:02d} min"
        ).format(
            hours=(duration // 60), minutes=(duration % 60)
        )
    # end def

    def init_document (self, **kw):
        self.w_text = self.get_component("scenario").w_text
        self.stats = dict()
    # end def

    def init_elements (self):
        super().init_elements()
        self.stats.clear()
    # end def

    def update_stats (self, text, tag):
        _s = self.stats
        _s["paragraph_count"] = (
            _s.setdefault("paragraph_count", 0) + 1
        )
        _s["word_count"] = (
            _s.setdefault("word_count", 0)
            + ((text or 0) and (text.count(" ") + 1))
        )
        _s["byte_count"] = (
            _s.setdefault("byte_count", 0) + len(text)
        )
        _s["scene_count"] = (
            _s.setdefault("scene_count", 0) + int(tag == "scene")
        )
        _s["dialogue_count"] = (
            _s.setdefault("dialogue_count", 0) + int(tag == "dialogue")
        )
        _s["transition_count"] = (
            _s.setdefault("transition_count", 0)
            + int(tag == "transition")
        )
        _s["movie_chars"] = _s.setdefault("movie_chars", set())
        if tag == "character":
            _s["movie_chars"].add(text)
        # end if
    # end def

# end class


class PDFDocumentStoryboard (PDFDocumentBase):

    def __init__ (self, doc_name, **kw):
        super().__init__(doc_name, **kw)
        self.scenario = self.get_component("scenario")
        self.storyboard = self.get_component("storyboard")
        self.w_text = self.scenario.w_text
        self.stats = dict()
    # end def

    def build_elements (self):
        self.reset_progress()
        exec("self.do_step_{}()".format(self.step))
    # end def

    def do_step_0 (self):
        self.reset_progress(force_reset=True)
        self.stats.clear()
        self.scene_lines = self.scenario.w_listbox.current_lines
        self.scene_ids = self.storyboard.listbox_scene.line_ids
        self.total_scenes = len(self.scene_lines)
        self.index = 0
        self.set_first_page_elements()
        self.step += 1
    # end def

    def do_step_1 (self):
        try:
            self.scene_shots = self.database.stb_get_scene_shots(
                self.scene_ids[self.index]
            )
        except:
            self.scene_shots = None
        else:
            self.progress = min(
                99.0, 100.0 * (self.index + 1) / self.total_scenes
            )
        # end try
        if not self.scene_shots:
            self.index += 1
            if self.index >= self.total_scenes:
                if self.stats.get("byte_count"):
                    self.step = 4
                else:
                    self.progress = 100
                # end if
            # end if
        else:
            _len_shots = len(self.scene_shots)
            self.stats["min_shot"] = min(
                self.stats.setdefault("min_shot", _len_shots),
                _len_shots
            )
            self.stats["max_shot"] = max(
                self.stats.setdefault("max_shot", 0), _len_shots
            )
            self.step += 1
        # end if
    # end def

    def do_step_2 (self):
        try:
            _start = float(self.scene_lines[self.index])
        except:
            self.progress = 100
        else:
            try:
                _end = float(self.scene_lines[self.index + 1])
            except:
                _end = "end"
            # end try
            _tagged_text = self.w_text.get_tagged_text(_start, _end)
            _only_texts = _tagged_text[::2]
            _only_tags = _tagged_text[1::2]
            if self.index:
                self.add_pagebreak()
            # end if
            self.add_paragraph(_("Scene preview"), self.styles["h2"])
            _print_left = self.options.get("print_shot_left")
            _print_right = self.options.get("print_shot_right")
            for _index, _text in enumerate(_only_texts):
                _text = _text.strip()
                try:
                    _style = self.styles[_only_tags[_index][0]]
                except:
                    _style = self.styles["body"]
                # end try
                for _line in _text.split("\n"):
                    _line = _line.strip()
                    if _style.name == "scene":
                        self.elements.append(
                            SeqNumberParagraph(
                                _line, _style,
                                print_left=_print_left,
                                print_right=_print_right,
                            )
                        )
                    else:
                        self.add_paragraph(_line, _style)
                    # end if
                    self.update_stats(_line, _style.name)
                # end for
            # end for
            self.add_paragraph(_("Scene shots"), self.styles["h2"])
            self.step += 1
        # end try
    # end def

    def do_step_3 (self):
        try:
            _row = self.scene_shots.pop(0)
        except:
            self.index += 1
            self.step = 1
        else:
            self.elements.append(
                SeqNumberParagraph(
                    _row["title"], self.styles["shot_title"],
                    print_left=self.options.get("print_shot_left"),
                    print_right=self.options.get("print_shot_right"),
                    use_subcounter=True,
                )
            )
            self.update_stats(_row["title"], "shot_title")
            for _line in _row["text"].strip().split("\n"):
                self.add_paragraph(_line, self.styles["shot_body"])
                self.update_stats(_line, "shot_body")
            # end for
        # end try
    # end def

    def do_step_4 (self):
        _s = self.stats
        _elements = (
            (_("Statistics"), "h1"),
            (_("Document"), "h2"),
            (
                PageNumber(_("Pages: {field}"), self.styles["stats"]),
                "*"
            ),
            (_("Paragraphs: {paragraph_count}").format(**_s), ""),
            (_("Words: {word_count}").format(**_s), ""),
            (
                _("Glyphs (letters/signs): {byte_count}").format(**_s),
                ""
            ),
            (_("Scenario"), "h2"),
            (_("Scenes with shots: {scene_count}").format(**_s), ""),
            (_("Dialogues: {dialogue_count}").format(**_s), ""),
            (_("Transitions: {transition_count}").format(**_s), ""),
            (_("Storyboard"), "h2"),
            (_("Total shots: {shot_count}").format(**_s), ""),
            (
                _("Average number of shots per scene: {:01.3f}")
                .format(
                    _s["shot_count"] / _s["scene_count"]
                    if _s["scene_count"] else 0.0
                ),
                ""
            ),
            (_("Min. shots per scene: {min_shot}").format(**_s), ""),
            (_("Max. shots per scene: {max_shot}").format(**_s), ""),
            #~ (_("").format(), ""),
        )
        self.add_pagebreak()
        for (_p, _s) in _elements:
            if _s == "*":
                self.elements.append(_p)
            else:
                self.add_paragraph(_p, self.styles[_s or "stats"])
            # end if
        # end for
        self.progress = 100
    # end def

    def update_stats (self, text, tag):
        _s = self.stats
        _s["paragraph_count"] = (
            _s.setdefault("paragraph_count", 0) + 1
        )
        _s["word_count"] = (
            _s.setdefault("word_count", 0)
            + ((text or 0) and (text.count(" ") + 1))
        )
        _s["byte_count"] = (
            _s.setdefault("byte_count", 0) + len(text)
        )
        _s["scene_count"] = (
            _s.setdefault("scene_count", 0) + int(tag == "scene")
        )
        _s["dialogue_count"] = (
            _s.setdefault("dialogue_count", 0) + int(tag == "dialogue")
        )
        _s["transition_count"] = (
            _s.setdefault("transition_count", 0)
            + int(tag == "transition")
        )
        _s["shot_count"] = (
            _s.setdefault("shot_count", 0) + int(tag == "shot_title")
        )
    # end def

# end class


class PageNumber (Paragraph):

    def __init__(self, text, style, bulletText=None, frags=None,
    caseSensitive=1, encoding='utf8', adjust=0, formatter=None):
        self.field_adjust = adjust
        self.field_formatter = formatter
        super().__init__(
            text, style, bulletText, frags, caseSensitive, encoding
        )
    # end def

    def draw (self):
        _field = "{field}"
        _formatter = self.field_formatter or (lambda t:str(t))
        for _i, _line in enumerate(self.blPara.lines):
            _frags = _line[1]
            if _field in _frags:
                _frags[_frags.index(_field)] = _formatter(
                    self.canv._doctemplate.page + self.field_adjust
                )
                break
            # end if
        # end for
        super().draw()
    # end def

# end class


class SeqNumberParagraph (Paragraph):

    MAIN_COUNTER = 1
    SUB_COUNTER = 1

    def __init__(self, text, style, bulletText=None, frags=None,
                caseSensitive=1, encoding='utf8', print_left=True,
                print_right=True, use_subcounter=False):
        self.options = {
            "print_left": bool(print_left),
            "print_right": bool(print_right),
            "use_subcounter": bool(use_subcounter),
        }
        super().__init__(
            text, style, bulletText, frags, caseSensitive, encoding
        )
    # end def

    def draw (self):
        self.draw_option_marks()
        super().draw()
    # end def

    def draw_option_marks (self):
        from reportlab.lib.units import inch
        _canvas = self.canv
        _text = self.get_seq_mark()
        _padding = 0.1*inch
        _font_size = self.blPara.fontSize
        _dy = self.height - _font_size
        self.update_seq_count()
        _canvas.saveState()
        _canvas.setFont("Courier-Bold", _font_size)
        if self.options["print_left"]:
            _canvas.drawRightString(-_padding, _dy, _text)
        # end if
        if self.options["print_right"]:
            _canvas.drawString(self.width + _padding, _dy, _text)
        # end if
        _canvas.restoreState()
    # end def

    def get_seq_mark (self):
        if self.options["use_subcounter"]:
            return (
                "#{}.{:02d}"
                .format(self.MAIN_COUNTER - 1, self.SUB_COUNTER)
            )
        else:
            return "#{}".format(self.MAIN_COUNTER)
        # end if
    # end def

    @classmethod
    def reset (cls):
        cls.MAIN_COUNTER = 1
        cls.SUB_COUNTER = 1
    # end def

    def update_seq_count (self):
        if self.options["use_subcounter"]:
            __class__.SUB_COUNTER += 1
        else:
            __class__.MAIN_COUNTER += 1
            __class__.SUB_COUNTER = 1
        # end if
    # end def

# end class
