#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
    tkScenarist - screen writing made simpler

    Copyright (c) 2014+ Raphaël Seban <motus@laposte.net>

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.

    If not, see http://www.gnu.org/licenses/
"""

from tkinter import Toplevel


class SplashScreen (Toplevel):

    APP_NAME = "My Application"

    BG_COLOR = "grey95"

    DEFAULT_CONFIG = {
        "relief": "solid",
        "highlightthickness": 1,
        "highlightbackground": "grey50",
    }

    FG_COLOR1 = "royal blue"
    FG_COLOR2 = "grey20"

    FONT1 = "times 36 bold italic"
    FONT2 = "helvetica 9"

    INFO = "Loading application, please wait..."

    def __init__ (self, master=None, **kw):
        super().__init__(master)
        self.initialize(**kw)
    # end def

    def destroy (self):
        self.hide()
        super().destroy()
    # end def

    def hide (self, *args, **kw):
        self.withdraw()
    # end def

    def initialize (self, **kw):
        self.hide()
        self.overrideredirect(True)
        self.bind("<Button-1>", self.hide)
        _cfg = self.DEFAULT_CONFIG.copy()
        for _key in set(kw).intersection(self.configure()):
            _cfg[_key] = kw[_key]
        # end for
        self.configure(**_cfg)
        self.setup_gui(**kw)
        self.update_idletasks()
        _w, _h = self.winfo_reqwidth(), self.winfo_reqheight()
        self.geometry(
            "{w}x{h}+{x}+{y}"
            .format(
                w=_w, h=_h,
                x=(self.winfo_screenwidth() - _w)//2,
                y=(self.winfo_screenheight() - _h)//2,
            )
        )
    # end def

    def setup_gui (self, *args, **kw):
        from tkinter import Frame, Label
        _bg = kw.get("bg") or kw.get("background") or self.BG_COLOR
        _frame = Frame(self, background=_bg)
        Label(
            _frame,
            anchor=kw.get("anchor1"),
            background=_bg,
            fg=kw.get("fg1") or kw.get("fgcolor1") or self.FG_COLOR1,
            font=kw.get("font1") or self.FONT1,
            justify=kw.get("justify1"),
            text=kw.get("name") or kw.get("app_name") or self.APP_NAME,
        ).pack(expand=1, fill="x", padx=20)
        try:
            _text = _(self.INFO)
        except:
            _text = self.INFO
        # end try
        _text = kw.get("info") or _text
        Label(
            _frame,
            anchor=kw.get("anchor2"),
            background=_bg,
            fg=kw.get("fg2") or kw.get("fgcolor2") or self.FG_COLOR2,
            font=kw.get("font2") or self.FONT2,
            justify=kw.get("justify2"),
            text=_text,
        ).pack(expand=1, fill="x", padx=20, pady=10)
        _frame.pack()
    # end def

    def show (self, *args, **kw):
        self.deiconify()
        self.lift()
        self.update()
    # end def

# end class
