#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""tkRAD lightweight Canvas widget module.

Implements:

* class RADCanvas -- tkRAD extended tkinter.Canvas class.

Example::

    import tkinter as tk
    from tkRAD.widgets.rad_canvas import RADCanvas
    root = tk.Tk()
    # you can mix tkinter.Canvas init options with tkRAD init options
    # unsupported options will be merely ignored
    my_canvas = RADCanvas(
        root,
        width=320,
        height=240,
        rc_dir="~/.config/testing",
        rc_file="canvas.rc",
        relief=tk.SUNKEN,
        background="white",
        borderwidth=3,
    )
    my_canvas.pack(expand=1, fill=tk.BOTH, padx=5, pady=5)
    root.mainloop()

*NEW* since **tkRAD v1.7**:

* from now, each subclass of mixin RADWidgetBase *MUST* call
  self.initialize() hook method by itself at coder's choice for calling
  point;
* tried to keep as much as possible backward compatibility with older
  versions of tkRAD.

Please, feel free to post an issue ticket on GitHub's project
bugtracker if you face any troubleshooting with this new version of
tkRAD's library.

GitHub's project bugtracker: https://github.com/tarball69/tkRAD/issues
(requires a GitHub member account to proceed).
"""

"""
    tkRAD - tkinter Rapid Application Development library

    (c) 2013+ Raphaël SEBAN <motus@laposte.net>

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.

    If not, see: http://www.gnu.org/licenses/
"""

import tkinter as tk
from . import rad_widget_base as rwb


class RADCanvas (rwb.RADWidgetBase, tk.Canvas):
    """tkRAD extended tkinter.Canvas class."""

    CONFIG = dict(
        borderwidth=0, highlightthickness=0, relief=tk.FLAT,
    )

    def __init__ (self, master=None, **kw):
        self.CONFIG = self.CONFIG.copy()
        self.CONFIG.update(kw)
        tk.Canvas.__init__(self, master)
        self.configure(**self._only_tk(self.CONFIG))
        rwb.RADWidgetBase.__init__(self, **kw)
        self.initialize(**kw)
    # end def

# end class
