#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""tkRAD Toplevel window module.

Implements:

* class RADWindow -- tkRAD extended tkinter.Toplevel class.

Example 1::

    import tkRAD
    tkRAD.RADWindow().run()

Example 2::

    import tkRAD
    tkRAD.RADWindow(hide_tk=True).run()

Example 3::

    import tkinter as tk
    import tkRAD

    class MyToolWindow (tkRAD.RADWindow):
        def on_close_window (self, *args, **kw):
            self.destroy()
        # end def
    # end class

    def create_window ():
        MyToolWindow(root)
    # end def

    root = tk.Tk()
    tk.Button(
        root, text="Click here", command=create_window
    ).pack(padx=20, pady=20)
    root.mainloop()

*NEW* since **tkRAD v1.7**:

* added new 'hide_tk' keyword argument on __init__(): set to True, this
  parameter forces withdrawing of Tk's Toplevel window;
* from now, each subclass of mixin RADWidgetBase *MUST* call
  self.initialize() hook method by itself;
* moved 'WM_DELETE_WINDOW' protocol to new *self.on_close_window()*
  event handler as it is more relevant than specific
  *self.on_quit_app()* event handler i.e. app secundary Toplevel()
  windows do *NOT* have to quit application, but merely close window;
* from now, window saves its geometry and window state ('normal',
  'maximized') at the very end, just before closing;
* tried to keep as much as possible backward compatibility with older
  versions of tkRAD.

Please, feel free to post an issue ticket on GitHub's project
bugtracker if you face any troubleshooting with this new version of
tkRAD's library.

GitHub's project bugtracker: https://github.com/tarball69/tkRAD/issues
(requires a GitHub member account to proceed).
"""

"""
    tkRAD - tkinter Rapid Application Development library

    (c) 2013+ Raphaël SEBAN <motus@laposte.net>

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.

    If not, see: http://www.gnu.org/licenses/
"""

import tkinter as tk
from . import rad_window_base as rwb


class RADWindow (rwb.RADWindowBase, tk.Toplevel):
    """tkRAD extended tkinter.Toplevel class."""

    CONFIG = dict()

    WINDOW_ID = "window"

    def __init__ (self, master=None, **kw):
        self.CONFIG = self.CONFIG.copy()
        self.CONFIG.update(kw)
        tk.Toplevel.__init__(self, master)
        self.configure(**self._only_tk(self.CONFIG))
        if kw.get("hide_tk"):
            tk._default_root.withdraw()
        # end if
        rwb.RADWindowBase.__init__(self, **kw)
        self.initialize(**kw)
    # end def

# end class
