#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""tkRAD XML attributes mapping module.

Implements:

* class RADXMLAttributesDict -- mapping dict() of RADXMLAttribute items
  for tkRAD parsers.

This module is intended to be used internally by tkRAD parsers only.

*NEW* since **tkRAD v1.7**:

Please, feel free to post an issue ticket on GitHub's project
bugtracker if you face any troubleshooting with this new version of
tkRAD's library.

GitHub's project bugtracker: https://github.com/tarball69/tkRAD/issues
(requires a GitHub member account to proceed).
"""

"""
    tkRAD - tkinter Rapid Application Development library

    (c) 2013+ Raphaël SEBAN <motus@laposte.net>

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.

    If not, see: http://www.gnu.org/licenses/
"""

from ..core.struct_dict import StructDict
from .rad_xml_attribute import RADXMLAttribute


class RADXMLAttributesDict (StructDict):
    """Mapping dict() of RADXMLAttribute items for tkRAD parsers."""

    def __init__ (self, *args, **kw):
        """Class constructor."""
        super().__init__(*args, **kw)
        self.item_type = RADXMLAttribute
        self.item_value_getter = "get_value"
        self.item_value_setter = "set_value"
    # end def

# end class
