#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""XML factory Frame widget module.

Implements:

* class RADXMLFrame -- XML factory general purpose widgets container
  class.

Example::

    # using RADXMLFrame as a tkRAD external context component
    import tkinter as tk
    import tkRAD
    xml_source = \"""
        <tkwidget>
            <label text="Hello, good people!" layout="pack"/>
            <button text="Quit" command="quit" layout="pack"/>
        </tkwidget>
    \"""
    root = tk.Tk()
    frame = tkRAD.RADXMLFrame(root)
    frame.xml_build(xml_source)
    frame.pack()
    root.mainloop()

*NEW* since **tkRAD v1.7**:

Please, feel free to post an issue ticket on GitHub's project
bugtracker if you face any troubleshooting with this new version of
tkRAD's library.

GitHub's project bugtracker: https://github.com/tarball69/tkRAD/issues
(requires a GitHub member account to proceed).
"""

"""
    tkRAD - tkinter Rapid Application Development library

    (c) 2013+ Raphaël SEBAN <motus@laposte.net>

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.

    If not, see: http://www.gnu.org/licenses/
"""

from tkinter import ttk
from . import rad_xml_widget as rxw


class RADXMLFrame (rxw.RADXMLWidget, ttk.Frame):
    """XML factory general purpose widgets container class."""

    CONFIG = dict()

    def __init__ (self, master=None, **kw):
        self.tk_parent = master
        self.CONFIG = self.CONFIG.copy()
        self.CONFIG.update(kw)
        ttk.Frame.__init__(self, master)
        self.configure(**self._only_tk(self.CONFIG))
        rxw.RADXMLWidget.__init__(self, tk_owner=self, **kw)
        self.initialize(**kw)
    # end def

    def initialize (self, *args, **kw):
        """Initialize current widget - hook method.

        Parameters:
            args -- variable list of arguments.

            kw -- keyword arguments.

        This method is a coding helper and should be called immediately
        after '__init__()' method.

        Current implementation does nothing.

        Feel free to extend or to override this hook method to best
        meet your own needs.

        *NEW* since **tkRAD v1.7**.

        *NEW* since **tkRAD v1.7.5**:

        * removed annoying calls to deprecated *init_widget()* method.
        """
        pass
    # end def

# end class
