#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""XML factory toplevel window widget module.

Implements:

* class RADXMLWindow -- XML factory toplevel window class.

Example::

    import tkRAD
    xml_source = \"""
        <tkwidget>
            <label
                text="Hello, good people!"
                layout="pack"
                resizable="yes"
            />
            <button
                text="Quit"
                command="@quit"
                layout="pack"
            />
        </tkwidget>
    \"""
    w = tkRAD.RADXMLWindow(hide_tk=True)
    w.xml_build(xml_source)
    w.run()

*NEW* since **tkRAD v1.7**:

Please, feel free to post an issue ticket on GitHub's project
bugtracker if you face any troubleshooting with this new version of
tkRAD's library.

GitHub's project bugtracker: https://github.com/tarball69/tkRAD/issues
(requires a GitHub member account to proceed).
"""

"""
    tkRAD - tkinter Rapid Application Development library

    (c) 2013+ Raphaël SEBAN <motus@laposte.net>

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.

    If not, see: http://www.gnu.org/licenses/
"""

from ..widgets.rad_window import RADWindow
from .rad_xml_frame import RADXMLFrame


class RADXMLWindow (RADWindow):
    """XML factory toplevel window class."""

    def initialize (self, *args, **kw):
        """Initialize current widget - hook method.

        Parameters:
            args -- variable list of arguments.

            kw -- keyword arguments.

        Feel free to extend or to override this hook method to best
        meet your own needs.

        *NEW* since **tkRAD v1.7**.
        """
        super().initialize(*args, **kw)
        self.h_owner = kw.pop("h_owner", self)
        self.mainframe = (
            kw.get("mainframe") or
            RADXMLFrame(self, h_owner=self.h_owner, **kw)
        )
        self.mainframe.xml_filename = (
            kw.get("xml_filename") or "window"
        )
    # end def

# end class
